mirror of https://lore.kernel.org/lkml/
 help / color / mirror / Atom feed
From: David Howells <dhowells@redhat.com>
To: linux-am33-list@redhat.com
Cc: linux-kernel@vger.kernel.org,
	Akira Takeuchi <takeuchi.akr@jp.panasonic.com>,
	Kiyoshi Owada <owada.kiyoshi@jp.panasonic.com>
Subject: [PATCH 20/43] MN10300: SMP: Differentiate local cache flushing
Date: Tue, 26 Oct 2010 03:54:48 +0100	[thread overview]
Message-ID: <20101026025448.23512.94942.stgit@warthog.procyon.org.uk> (raw)
In-Reply-To: <20101026025301.23512.24525.stgit@warthog.procyon.org.uk>

From: Akira Takeuchi <takeuchi.akr@jp.panasonic.com>

Differentiate local cache flushing from global cache flushing so that they can
be done differently on SMP systems.

Rename the cache functions from:

	mn10300_[id]cache_*()

to:

	mn10300_[id]_localcache_*()

and on a UP system, assign the global labels to the local labels.

Signed-off-by: Akira Takeuchi <takeuchi.akr@jp.panasonic.com>
Signed-off-by: Kiyoshi Owada <owada.kiyoshi@jp.panasonic.com>
Signed-off-by: David Howells <dhowells@redhat.com>
---

 arch/mn10300/include/asm/cacheflush.h            |   50 ++++++
 arch/mn10300/mm/cache-flush-by-tag.S             |  143 +++++++++++-----
 arch/mn10300/mm/cache-inv-by-tag.S               |  199 ++++++++++++++--------
 arch/mn10300/proc-mn103e010/include/proc/cache.h |    9 +
 4 files changed, 289 insertions(+), 112 deletions(-)


diff --git a/arch/mn10300/include/asm/cacheflush.h b/arch/mn10300/include/asm/cacheflush.h
index a9f4168..748143f 100644
--- a/arch/mn10300/include/asm/cacheflush.h
+++ b/arch/mn10300/include/asm/cacheflush.h
@@ -20,12 +20,31 @@
  * Primitive routines
  */
 #ifdef CONFIG_MN10300_CACHE_ENABLED
+extern void mn10300_local_icache_inv(void);
+extern void mn10300_local_icache_inv_page(unsigned long start);
+extern void mn10300_local_icache_inv_range(unsigned long start, unsigned long end);
+extern void mn10300_local_icache_inv_range2(unsigned long start, unsigned long size);
+extern void mn10300_local_dcache_inv(void);
+extern void mn10300_local_dcache_inv_page(unsigned long start);
+extern void mn10300_local_dcache_inv_range(unsigned long start, unsigned long end);
+extern void mn10300_local_dcache_inv_range2(unsigned long start, unsigned long size);
 extern void mn10300_icache_inv(void);
+extern void mn10300_icache_inv_page(unsigned long start);
+extern void mn10300_icache_inv_range(unsigned long start, unsigned long end);
+extern void mn10300_icache_inv_range2(unsigned long start, unsigned long size);
 extern void mn10300_dcache_inv(void);
 extern void mn10300_dcache_inv_page(unsigned long start);
 extern void mn10300_dcache_inv_range(unsigned long start, unsigned long end);
 extern void mn10300_dcache_inv_range2(unsigned long start, unsigned long size);
 #ifdef CONFIG_MN10300_CACHE_WBACK
+extern void mn10300_local_dcache_flush(void);
+extern void mn10300_local_dcache_flush_page(unsigned long start);
+extern void mn10300_local_dcache_flush_range(unsigned long start, unsigned long end);
+extern void mn10300_local_dcache_flush_range2(unsigned long start, unsigned long size);
+extern void mn10300_local_dcache_flush_inv(void);
+extern void mn10300_local_dcache_flush_inv_page(unsigned long start);
+extern void mn10300_local_dcache_flush_inv_range(unsigned long start, unsigned long end);
+extern void mn10300_local_dcache_flush_inv_range2(unsigned long start, unsigned long size);
 extern void mn10300_dcache_flush(void);
 extern void mn10300_dcache_flush_page(unsigned long start);
 extern void mn10300_dcache_flush_range(unsigned long start, unsigned long end);
@@ -35,6 +54,18 @@ extern void mn10300_dcache_flush_inv_page(unsigned long start);
 extern void mn10300_dcache_flush_inv_range(unsigned long start, unsigned long end);
 extern void mn10300_dcache_flush_inv_range2(unsigned long start, unsigned long size);
 #else
+#define mn10300_local_dcache_flush()			do {} while (0)
+#define mn10300_local_dcache_flush_page(start)		do {} while (0)
+#define mn10300_local_dcache_flush_range(start, end)	do {} while (0)
+#define mn10300_local_dcache_flush_range2(start, size)	do {} while (0)
+#define mn10300_local_dcache_flush_inv() \
+		mn10300_local_dcache_inv()
+#define mn10300_local_dcache_flush_inv_page(start) \
+		mn10300_local_dcache_inv_page(start)
+#define mn10300_local_dcache_flush_inv_range(start, end) \
+		mn10300_local_dcache_inv_range(start, end)
+#define mn10300_local_dcache_flush_inv_range2(start, size) \
+		mn10300_local_dcache_inv_range2(start, size)
 #define mn10300_dcache_flush()				do {} while (0)
 #define mn10300_dcache_flush_page(start)		do {} while (0)
 #define mn10300_dcache_flush_range(start, end)		do {} while (0)
@@ -48,7 +79,26 @@ extern void mn10300_dcache_flush_inv_range2(unsigned long start, unsigned long s
 	mn10300_dcache_inv_range2((start), (size))
 #endif /* CONFIG_MN10300_CACHE_WBACK */
 #else
+#define mn10300_local_icache_inv()			do {} while (0)
+#define mn10300_local_icache_inv_page(start)		do {} while (0)
+#define mn10300_local_icache_inv_range(start, end)	do {} while (0)
+#define mn10300_local_icache_inv_range2(start, size)	do {} while (0)
+#define mn10300_local_dcache_inv()			do {} while (0)
+#define mn10300_local_dcache_inv_page(start)		do {} while (0)
+#define mn10300_local_dcache_inv_range(start, end)	do {} while (0)
+#define mn10300_local_dcache_inv_range2(start, size)	do {} while (0)
+#define mn10300_local_dcache_flush()			do {} while (0)
+#define mn10300_local_dcache_flush_inv_page(start)	do {} while (0)
+#define mn10300_local_dcache_flush_inv()		do {} while (0)
+#define mn10300_local_dcache_flush_inv_range(start, end)do {} while (0)
+#define mn10300_local_dcache_flush_inv_range2(start, size) do {} while (0)
+#define mn10300_local_dcache_flush_page(start)		do {} while (0)
+#define mn10300_local_dcache_flush_range(start, end)	do {} while (0)
+#define mn10300_local_dcache_flush_range2(start, size)	do {} while (0)
 #define mn10300_icache_inv()				do {} while (0)
+#define mn10300_icache_inv_page(start)			do {} while (0)
+#define mn10300_icache_inv_range(start, end)		do {} while (0)
+#define mn10300_icache_inv_range2(start, size)		do {} while (0)
 #define mn10300_dcache_inv()				do {} while (0)
 #define mn10300_dcache_inv_page(start)			do {} while (0)
 #define mn10300_dcache_inv_range(start, end)		do {} while (0)
diff --git a/arch/mn10300/mm/cache-inv-by-tag.S b/arch/mn10300/mm/cache-inv-by-tag.S
index e839d0a..e9713b4 100644
--- a/arch/mn10300/mm/cache-inv-by-tag.S
+++ b/arch/mn10300/mm/cache-inv-by-tag.S
@@ -13,40 +13,65 @@
 #include <asm/smp.h>
 #include <asm/page.h>
 #include <asm/cache.h>
+#include <asm/irqflags.h>
+#include <asm/cacheflush.h>
 
-#define mn10300_dcache_inv_range_intr_interval \
+#define mn10300_local_dcache_inv_range_intr_interval \
 	+((1 << MN10300_DCACHE_INV_RANGE_INTR_LOG2_INTERVAL) - 1)
 
-#if mn10300_dcache_inv_range_intr_interval > 0xff
+#if mn10300_local_dcache_inv_range_intr_interval > 0xff
 #error MN10300_DCACHE_INV_RANGE_INTR_LOG2_INTERVAL must be 8 or less
 #endif
 
 	.am33_2
 
-	.globl mn10300_icache_inv
-	.globl mn10300_dcache_inv
-	.globl mn10300_dcache_inv_range
-	.globl mn10300_dcache_inv_range2
-	.globl mn10300_dcache_inv_page
+	.globl	mn10300_local_icache_inv_page
+	.globl	mn10300_local_icache_inv_range
+	.globl	mn10300_local_icache_inv_range2
+
+mn10300_local_icache_inv_page	= mn10300_local_icache_inv
+mn10300_local_icache_inv_range	= mn10300_local_icache_inv
+mn10300_local_icache_inv_range2	= mn10300_local_icache_inv
+
+#ifndef CONFIG_SMP
+	.globl	mn10300_icache_inv
+	.globl	mn10300_icache_inv_page
+	.globl	mn10300_icache_inv_range
+	.globl	mn10300_icache_inv_range2
+	.globl	mn10300_dcache_inv
+	.globl	mn10300_dcache_inv_page
+	.globl	mn10300_dcache_inv_range
+	.globl	mn10300_dcache_inv_range2
+
+mn10300_icache_inv		= mn10300_local_icache_inv
+mn10300_icache_inv_page		= mn10300_local_icache_inv_page
+mn10300_icache_inv_range	= mn10300_local_icache_inv_range
+mn10300_icache_inv_range2	= mn10300_local_icache_inv_range2
+mn10300_dcache_inv		= mn10300_local_dcache_inv
+mn10300_dcache_inv_page		= mn10300_local_dcache_inv_page
+mn10300_dcache_inv_range	= mn10300_local_dcache_inv_range
+mn10300_dcache_inv_range2	= mn10300_local_dcache_inv_range2
+
+#endif /* !CONFIG_SMP */
 
 ###############################################################################
 #
-# void mn10300_icache_inv(void)
+# void mn10300_local_icache_inv(void)
 # Invalidate the entire icache
 #
 ###############################################################################
 	ALIGN
-mn10300_icache_inv:
+	.globl	mn10300_local_icache_inv
+        .type	mn10300_local_icache_inv,@function
+mn10300_local_icache_inv:
 	mov	CHCTR,a0
 
 	movhu	(a0),d0
 	btst	CHCTR_ICEN,d0
-	beq	mn10300_icache_inv_end
+	beq	mn10300_local_icache_inv_end
 
-	mov	epsw,d1
-	and	~EPSW_IE,epsw
-	nop
-	nop
+#if defined(CONFIG_AM33_2) || defined(CONFIG_AM33_3)
+	LOCAL_CLI_SAVE(d1)
 
 	# disable the icache
 	and	~CHCTR_ICEN,d0
@@ -75,29 +100,36 @@ mn10300_icache_inv:
 	movhu	d0,(a0)
 	movhu	(a0),d0
 
-	mov	d1,epsw
+	LOCAL_IRQ_RESTORE(d1)
+#else /* CONFIG_AM33_2 || CONFIG_AM33_3 */
+	# invalidate
+	or	CHCTR_ICINV,d0
+	movhu	d0,(a0)
+	movhu	(a0),d0
+#endif /* CONFIG_AM33_2 || CONFIG_AM33_3 */
 
-mn10300_icache_inv_end:
+mn10300_local_icache_inv_end:
 	ret	[],0
+	.size	mn10300_local_icache_inv,.-mn10300_local_icache_inv
 
 ###############################################################################
 #
-# void mn10300_dcache_inv(void)
+# void mn10300_local_dcache_inv(void)
 # Invalidate the entire dcache
 #
 ###############################################################################
 	ALIGN
-mn10300_dcache_inv:
+	.globl	mn10300_local_dcache_inv
+	.type	mn10300_local_dcache_inv,@function
+mn10300_local_dcache_inv:
 	mov	CHCTR,a0
 
 	movhu	(a0),d0
 	btst	CHCTR_DCEN,d0
-	beq	mn10300_dcache_inv_end
+	beq	mn10300_local_dcache_inv_end
 
-	mov	epsw,d1
-	and	~EPSW_IE,epsw
-	nop
-	nop
+#if defined(CONFIG_AM33_2) || defined(CONFIG_AM33_3)
+	LOCAL_CLI_SAVE(d1)
 
 	# disable the dcache
 	and	~CHCTR_DCEN,d0
@@ -126,40 +158,69 @@ mn10300_dcache_inv:
 	movhu	d0,(a0)
 	movhu	(a0),d0
 
-	mov	d1,epsw
+	LOCAL_IRQ_RESTORE(d1)
+#else /* CONFIG_AM33_2 || CONFIG_AM33_3 */
+	# invalidate
+	or	CHCTR_DCINV,d0
+	movhu	d0,(a0)
+	movhu	(a0),d0
+#endif /* CONFIG_AM33_2 || CONFIG_AM33_3 */
 
-mn10300_dcache_inv_end:
+mn10300_local_dcache_inv_end:
 	ret	[],0
+	.size	mn10300_local_dcache_inv,.-mn10300_local_dcache_inv
 
 ###############################################################################
 #
-# void mn10300_dcache_inv_range(unsigned start, unsigned end)
-# void mn10300_dcache_inv_range2(unsigned start, unsigned size)
-# void mn10300_dcache_inv_page(unsigned start)
+# void mn10300_local_dcache_inv_range(unsigned long start, unsigned long end)
+# void mn10300_local_dcache_inv_range2(unsigned long start, unsigned long size)
+# void mn10300_local_dcache_inv_page(unsigned long start)
 # Invalidate a range of addresses on a page in the dcache
 #
 ###############################################################################
 	ALIGN
-mn10300_dcache_inv_page:
+	.globl	mn10300_local_dcache_inv_page
+	.globl	mn10300_local_dcache_inv_range
+	.globl	mn10300_local_dcache_inv_range2
+	.type	mn10300_local_dcache_inv_page,@function
+	.type	mn10300_local_dcache_inv_range,@function
+	.type	mn10300_local_dcache_inv_range2,@function
+mn10300_local_dcache_inv_page:
+	and	~(PAGE_SIZE-1),d0
 	mov	PAGE_SIZE,d1
-mn10300_dcache_inv_range2:
+mn10300_local_dcache_inv_range2:
 	add	d0,d1
-mn10300_dcache_inv_range:
+mn10300_local_dcache_inv_range:
+	# If we are in writeback mode we check the start and end alignments,
+	# and if they're not cacheline-aligned, we must flush any bits outside
+	# the range that share cachelines with stuff inside the range
+#ifdef CONFIG_MN10300_CACHE_WBACK
+	btst	~(L1_CACHE_BYTES-1),d0
+	bne	1f
+	btst	~(L1_CACHE_BYTES-1),d1
+	beq	2f
+1:
+	bra	mn10300_local_dcache_flush_inv_range
+2:
+#endif /* CONFIG_MN10300_CACHE_WBACK */
+
 	movm	[d2,d3,a2],(sp)
-	mov	CHCTR,a2
 
+	mov	CHCTR,a2
 	movhu	(a2),d2
 	btst	CHCTR_DCEN,d2
-	beq	mn10300_dcache_inv_range_end
+	beq	mn10300_local_dcache_inv_range_end
 
+#ifndef CONFIG_MN10300_CACHE_WBACK
 	and	L1_CACHE_TAG_ADDRESS|L1_CACHE_TAG_ENTRY,d0	# round start
 								# addr down
-	mov	d0,a1
 
-	add	L1_CACHE_BYTES,d1			# round end addr up
+	add	L1_CACHE_BYTES,d1		# round end addr up
 	and	L1_CACHE_TAG_ADDRESS|L1_CACHE_TAG_ENTRY,d1
+#endif /* !CONFIG_MN10300_CACHE_WBACK */
+	mov	d0,a1
 
-	clr	d2				# we're going to clear tag ram
+	clr	d2				# we're going to clear tag RAM
 						# entries
 
 	# read the tags from the tag RAM, and if they indicate a valid dirty
@@ -176,14 +237,8 @@ mn10300_dcache_inv_range:
 
 	and	~(L1_CACHE_DISPARITY-1),a1	# determine comparator base
 
-mn10300_dcache_inv_range_outer_loop:
-	# disable interrupts
-	mov	epsw,d3
-	and	~EPSW_IE,epsw
-	nop					# note that reading CHCTR and
-						# AND'ing D0 occupy two delay
-						# slots after disabling
-						# interrupts
+mn10300_local_dcache_inv_range_outer_loop:
+	LOCAL_CLI_SAVE(d3)
 
 	# disable the dcache
 	movhu	(a2),d0
@@ -196,63 +251,63 @@ mn10300_dcache_inv_range_outer_loop:
 	btst	CHCTR_DCBUSY,d0
 	lne
 
-mn10300_dcache_inv_range_loop:
+mn10300_local_dcache_inv_range_loop:
 
 	# process the way 0 slot
 	mov	(L1_CACHE_WAYDISP*0,a0),d0	# read the tag in the way 0 slot
 	btst	L1_CACHE_TAG_VALID,d0
-	beq	mn10300_dcache_inv_range_skip_0	# jump if this cacheline is not
-						# valid
+	beq	mn10300_local_dcache_inv_range_skip_0	# jump if this cacheline
+						# is not valid
 
 	xor	a1,d0
 	lsr	12,d0
-	bne	mn10300_dcache_inv_range_skip_0	# jump if not this cacheline
+	bne	mn10300_local_dcache_inv_range_skip_0	# jump if not this cacheline
 
-	mov	d2,(a0)				# kill the tag
+	mov	d2,(L1_CACHE_WAYDISP*0,a0)	# kill the tag
 
-mn10300_dcache_inv_range_skip_0:
+mn10300_local_dcache_inv_range_skip_0:
 
 	# process the way 1 slot
 	mov	(L1_CACHE_WAYDISP*1,a0),d0	# read the tag in the way 1 slot
 	btst	L1_CACHE_TAG_VALID,d0
-	beq	mn10300_dcache_inv_range_skip_1	# jump if this cacheline is not
-						# valid
+	beq	mn10300_local_dcache_inv_range_skip_1	# jump if this cacheline
+						# is not valid
 
 	xor	a1,d0
 	lsr	12,d0
-	bne	mn10300_dcache_inv_range_skip_1	# jump if not this cacheline
+	bne	mn10300_local_dcache_inv_range_skip_1	# jump if not this cacheline
 
-	mov	d2,(a0)				# kill the tag
+	mov	d2,(L1_CACHE_WAYDISP*1,a0)	# kill the tag
 
-mn10300_dcache_inv_range_skip_1:
+mn10300_local_dcache_inv_range_skip_1:
 
 	# process the way 2 slot
 	mov	(L1_CACHE_WAYDISP*2,a0),d0	# read the tag in the way 2 slot
 	btst	L1_CACHE_TAG_VALID,d0
-	beq	mn10300_dcache_inv_range_skip_2	# jump if this cacheline is not
-						# valid
+	beq	mn10300_local_dcache_inv_range_skip_2	# jump if this cacheline
+						# is not valid
 
 	xor	a1,d0
 	lsr	12,d0
-	bne	mn10300_dcache_inv_range_skip_2	# jump if not this cacheline
+	bne	mn10300_local_dcache_inv_range_skip_2	# jump if not this cacheline
 
-	mov	d2,(a0)				# kill the tag
+	mov	d2,(L1_CACHE_WAYDISP*2,a0)	# kill the tag
 
-mn10300_dcache_inv_range_skip_2:
+mn10300_local_dcache_inv_range_skip_2:
 
 	# process the way 3 slot
 	mov	(L1_CACHE_WAYDISP*3,a0),d0	# read the tag in the way 3 slot
 	btst	L1_CACHE_TAG_VALID,d0
-	beq	mn10300_dcache_inv_range_skip_3	# jump if this cacheline is not
-						# valid
+	beq	mn10300_local_dcache_inv_range_skip_3	# jump if this cacheline
+						# is not valid
 
 	xor	a1,d0
 	lsr	12,d0
-	bne	mn10300_dcache_inv_range_skip_3	# jump if not this cacheline
+	bne	mn10300_local_dcache_inv_range_skip_3	# jump if not this cacheline
 
-	mov	d2,(a0)				# kill the tag
+	mov	d2,(L1_CACHE_WAYDISP*3,a0)	# kill the tag
 
-mn10300_dcache_inv_range_skip_3:
+mn10300_local_dcache_inv_range_skip_3:
 
 	# approx every N steps we re-enable the cache and see if there are any
 	# interrupts to be processed
@@ -260,9 +315,10 @@ mn10300_dcache_inv_range_skip_3:
 	# (the bottom nibble of the count is zero in both cases)
 	add	L1_CACHE_BYTES,a0
 	add	L1_CACHE_BYTES,a1
+	and	~L1_CACHE_WAYDISP,a0
 	add	-1,d1
-	btst	mn10300_dcache_inv_range_intr_interval,d1
-	bne	mn10300_dcache_inv_range_loop
+	btst	mn10300_local_dcache_inv_range_intr_interval,d1
+	bne	mn10300_local_dcache_inv_range_loop
 
 	# wait for the cache to finish what it's doing
 	setlb
@@ -279,11 +335,14 @@ mn10300_dcache_inv_range_skip_3:
 	# - we don't bother with delay NOPs as we'll have enough instructions
 	#   before we disable interrupts again to give the interrupts a chance
 	#   to happen
-	mov	d3,epsw
+	LOCAL_IRQ_RESTORE(d3)
 
 	# go around again if the counter hasn't yet reached zero
 	add	0,d1
-	bne	mn10300_dcache_inv_range_outer_loop
+	bne	mn10300_local_dcache_inv_range_outer_loop
 
-mn10300_dcache_inv_range_end:
+mn10300_local_dcache_inv_range_end:
 	ret	[d2,d3,a2],12
+	.size	mn10300_local_dcache_inv_page,.-mn10300_local_dcache_inv_page
+	.size	mn10300_local_dcache_inv_range,.-mn10300_local_dcache_inv_range
+	.size	mn10300_local_dcache_inv_range2,.-mn10300_local_dcache_inv_range2
diff --git a/arch/mn10300/proc-mn103e010/include/proc/cache.h b/arch/mn10300/proc-mn103e010/include/proc/cache.h
index bdc1f9a..c152800 100644
--- a/arch/mn10300/proc-mn103e010/include/proc/cache.h
+++ b/arch/mn10300/proc-mn103e010/include/proc/cache.h
@@ -30,4 +30,13 @@
  */
 #define MN10300_DCACHE_INV_RANGE_INTR_LOG2_INTERVAL	4
 
+/*
+ * The size of range at which it becomes more economical to just flush the
+ * whole cache rather than trying to flush the specified range.
+ */
+#define MN10300_DCACHE_FLUSH_BORDER	\
+	+(L1_CACHE_NWAYS * L1_CACHE_NENTRIES * L1_CACHE_BYTES)
+#define MN10300_DCACHE_FLUSH_INV_BORDER	\
+	+(L1_CACHE_NWAYS * L1_CACHE_NENTRIES * L1_CACHE_BYTES)
+
 #endif /* _ASM_PROC_CACHE_H */


  parent reply	other threads:[~2010-10-26  2:55 UTC|newest]

Thread overview: 47+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2010-10-26  2:53 [PATCH 00/43] MN10300: Add SMP support and AM34 CPU core support David Howells
2010-10-26  2:53 ` [PATCH 01/43] Partially revert patch that encloses asm-offset.h numbers in brackets David Howells
2010-10-26  2:53 ` [PATCH 02/43] MN10300: BUG to BUG_ON changes David Howells
2010-10-26  2:53 ` [PATCH 03/43] bitops: Change the bitmap index from int to unsigned long [mn10300] David Howells
2010-10-26  2:53 ` [PATCH 04/43] mn10300: Use pci_claim_resource David Howells
2010-10-26  2:53 ` [PATCH 05/43] MN10300: Prevent cnt32_to_63() from being preempted in sched_clock() David Howells
2010-10-26  2:53 ` [PATCH 06/43] Typedef SMP call function pointer David Howells
2010-10-26  2:53 ` [PATCH 07/43] MN10300: Don't cast away the volatile in test_bit() David Howells
2010-10-26  2:53 ` [PATCH 08/43] MN10300: Add reads[bwl]() and writes[bwl]() David Howells
2010-10-26  2:53 ` [PATCH 09/43] MN10300: Provide the functions to fully disable maskable interrupts David Howells
2010-10-26  2:53 ` [PATCH 10/43] MN10300: Differentiate AM33_2 and AM33_3 in config David Howells
2010-10-26  2:54 ` [PATCH 11/43] MN10300: Move DMA engine control reg defs to MN103E010 processor directory David Howells
2010-10-26  2:54 ` [PATCH 12/43] MN10300: Don't hard code the cacheline size in register defs David Howells
2010-10-26  2:54 ` [PATCH 13/43] MN10300: Add CPU register bits for AM34 David Howells
2010-10-26  2:54 ` [PATCH 14/43] MN10300: Remove monitor/JTAG functions David Howells
2010-10-26  2:54 ` [PATCH 15/43] MN10300: Cache: Split cache bits out of arch Kconfig David Howells
2010-10-26  2:54 ` [PATCH 16/43] MN10300: Provide a MN10300_CACHE_ENABLED config option David Howells
2010-10-26  2:54 ` [PATCH 17/43] MN10300: Reorder asm/cacheflush.h to put primitives first David Howells
2010-10-26  2:54 ` [PATCH 18/43] MN10300: AM34: The current cacheflush routines operate by controlling tag regs David Howells
2010-10-26  2:54 ` [PATCH 19/43] MN10300: Cacheflush functions should take unsigned long addresses David Howells
2010-10-26  2:54 ` David Howells [this message]
2010-10-26  2:54 ` [PATCH 21/43] MN10300: AM34: Add cacheflushing by using the AM34 purge registers David Howells
2010-10-26  2:54 ` [PATCH 22/43] MN10300: Allow some cacheflushes to be avoided if cache snooping is available David Howells
2010-10-26  2:55 ` [PATCH 23/43] MN10300: Cache: Implement SMP global cache flushing David Howells
2010-10-26  2:55 ` [PATCH 24/43] MN10300: Make the boot wrapper able to use writeback caching David Howells
2010-10-26  2:55 ` [PATCH 25/43] MN10300: AM34 erratum requires MMUCTR read and write on exception entry David Howells
2010-10-26  2:55 ` [PATCH 26/43] MN10300: Rename __flush_tlb*() to local_flush_tlb*() David Howells
2010-10-26  2:55 ` [PATCH 27/43] MN10300: Make the use of PIDR to mark TLB entries controllable David Howells
2010-10-26  2:55 ` [PATCH 28/43] MN10300: Use the [ID]PTEL2 registers rather than [ID]PTEL for TLB control David Howells
2010-10-26  2:55 ` [PATCH 29/43] MN10300: SMP TLB flushing David Howells
2010-10-26  2:55 ` [PATCH 30/43] MN10300: Make the FPU operate in non-lazy mode under SMP David Howells
2010-10-26  2:55 ` [PATCH 31/43] MN10300: Implement atomic ops using atomic ops unit David Howells
2010-10-26  2:55 ` [PATCH 32/43] MN10300: Optimise do_csum() David Howells
2010-10-26  2:55 ` [PATCH 33/43] MN10300: Make various interrupt priority settings configurable David Howells
2010-10-26  2:56 ` [PATCH 34/43] MN10300: Delete idle_timestamp from irq_cpustat_t David Howells
2010-10-26  2:56 ` [PATCH 35/43] MN10300: And Panasonic AM34 subarch and implement SMP David Howells
2010-10-27 13:31   ` [Linux-am33-list] " Suzuki Takashi
2010-10-27 14:33   ` David Howells
2010-10-26  2:56 ` [PATCH 36/43] MN10300: ASB2364: Add support for SMSC911X and SMC911X David Howells
2010-10-26  2:56 ` [PATCH 37/43] MN10300: Map userspace atomic op regs as a vmalloc page David Howells
2010-10-26  2:56 ` [PATCH 38/43] MN10300: Specify an ELF HWCAP flag for MN10300 Atomic Operations Unit support David Howells
2010-10-26  2:56 ` [PATCH 39/43] MN10300: Generic time support David Howells
2010-10-26  2:56 ` [PATCH 40/43] MN10300: Update the ASB2303 defconfig David Howells
2010-10-26  2:56 ` [PATCH 41/43] MN10300: Create a defconfig for the ASB2364 board David Howells
2010-10-26  2:56 ` [PATCH 42/43] MN10300: Change "Matsushita" to "Panasonic" David Howells
2010-10-26  2:56 ` [PATCH 43/43] MN10300: Save frame pointer in thread_info struct rather than global var David Howells
2010-10-26  3:07 ` [PATCH 00/43] MN10300: Add SMP support and AM34 CPU core support David Howells

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20101026025448.23512.94942.stgit@warthog.procyon.org.uk \
    --to=dhowells@redhat.com \
    --cc=linux-am33-list@redhat.com \
    --cc=linux-kernel@vger.kernel.org \
    --cc=owada.kiyoshi@jp.panasonic.com \
    --cc=takeuchi.akr@jp.panasonic.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox

all inboxes | Powered by JetHome®
diff --git a/arch/mn10300/mm/cache-flush-by-tag.S b/arch/mn10300/mm/cache-flush-by-tag.S
index 8fe90e4..5cd6a27 100644
--- a/arch/mn10300/mm/cache-flush-by-tag.S
+++ b/arch/mn10300/mm/cache-flush-by-tag.S
@@ -1,4 +1,4 @@
-/* MN10300 CPU core caching routines
+/* MN10300 CPU core caching routines, using direct tag flushing
  *
  * Copyright (C) 2007 Red Hat, Inc. All Rights Reserved.
  * Written by David Howells (dhowells@redhat.com)
@@ -14,8 +14,11 @@
 #include <asm/smp.h>
 #include <asm/page.h>
 #include <asm/cache.h>
+#include <asm/irqflags.h>
 
 	.am33_2
+
+#ifndef CONFIG_SMP
 	.globl mn10300_dcache_flush
 	.globl mn10300_dcache_flush_page
 	.globl mn10300_dcache_flush_range
@@ -25,17 +28,30 @@
 	.globl mn10300_dcache_flush_inv_range
 	.globl mn10300_dcache_flush_inv_range2
 
+mn10300_dcache_flush		= mn10300_local_dcache_flush
+mn10300_dcache_flush_page	= mn10300_local_dcache_flush_page
+mn10300_dcache_flush_range	= mn10300_local_dcache_flush_range
+mn10300_dcache_flush_range2	= mn10300_local_dcache_flush_range2
+mn10300_dcache_flush_inv	= mn10300_local_dcache_flush_inv
+mn10300_dcache_flush_inv_page	= mn10300_local_dcache_flush_inv_page
+mn10300_dcache_flush_inv_range	= mn10300_local_dcache_flush_inv_range
+mn10300_dcache_flush_inv_range2	= mn10300_local_dcache_flush_inv_range2
+
+#endif /* !CONFIG_SMP */
+
 ###############################################################################
 #
-# void mn10300_dcache_flush(void)
+# void mn10300_local_dcache_flush(void)
 # Flush the entire data cache back to RAM
 #
 ###############################################################################
 	ALIGN
-mn10300_dcache_flush:
+	.globl	mn10300_local_dcache_flush
+        .type	mn10300_local_dcache_flush,@function
+mn10300_local_dcache_flush:
 	movhu	(CHCTR),d0
 	btst	CHCTR_DCEN,d0
-	beq	mn10300_dcache_flush_end
+	beq	mn10300_local_dcache_flush_end
 
 	# read the addresses tagged in the cache's tag RAM and attempt to flush
 	# those addresses specifically
@@ -44,41 +60,56 @@ mn10300_dcache_flush:
 	mov	DCACHE_PURGE(0,0),a1		# dcache purge request address
 	mov	L1_CACHE_NWAYS*L1_CACHE_NENTRIES,d1  # total number of entries
 
-mn10300_dcache_flush_loop:
+mn10300_local_dcache_flush_loop:
 	mov	(a0),d0
 	and	L1_CACHE_TAG_ADDRESS|L1_CACHE_TAG_ENTRY,d0
 	or	L1_CACHE_TAG_VALID,d0		# retain valid entries in the
 						# cache
 	mov	d0,(a1)				# conditional purge
 
-mn10300_dcache_flush_skip:
 	add	L1_CACHE_BYTES,a0
 	add	L1_CACHE_BYTES,a1
 	add	-1,d1
-	bne	mn10300_dcache_flush_loop
+	bne	mn10300_local_dcache_flush_loop
 
-mn10300_dcache_flush_end:
+mn10300_local_dcache_flush_end:
 	ret	[],0
+	.size	mn10300_local_dcache_flush,.-mn10300_local_dcache_flush
 
 ###############################################################################
 #
 # void mn10300_dcache_flush_page(unsigned long start)
 # void mn10300_dcache_flush_range(unsigned long start, unsigned long end)
 # void mn10300_dcache_flush_range2(unsigned long start, unsigned long size)
 # Flush a range of addresses on a page in the dcache
 #
 ###############################################################################
 	ALIGN
-mn10300_dcache_flush_page:
+	.globl	mn10300_local_dcache_flush_page
+	.globl	mn10300_local_dcache_flush_range
+	.globl	mn10300_local_dcache_flush_range2
+	.type	mn10300_local_dcache_flush_page,@function
+	.type	mn10300_local_dcache_flush_range,@function
+	.type	mn10300_local_dcache_flush_range2,@function
+mn10300_local_dcache_flush_page:
+	and	~(PAGE_SIZE-1),d0
 	mov	PAGE_SIZE,d1
-mn10300_dcache_flush_range2:
+mn10300_local_dcache_flush_range2:
 	add	d0,d1
-mn10300_dcache_flush_range:
-	movm	[d2,d3],(sp)
+mn10300_local_dcache_flush_range:
+	movm	[d2],(sp)
 
 	movhu	(CHCTR),d2
 	btst	CHCTR_DCEN,d2
-	beq	mn10300_dcache_flush_range_end
+	beq	mn10300_local_dcache_flush_range_end
+
+	sub	d0,d1,a0
+	cmp	MN10300_DCACHE_FLUSH_BORDER,a0
+	ble	1f
+
+	movm	(sp),[d2]
+	bra	mn10300_local_dcache_flush
+1:
 
 	# round start addr down
 	and	L1_CACHE_TAG_ADDRESS|L1_CACHE_TAG_ENTRY,d0
@@ -101,7 +132,7 @@ mn10300_dcache_flush_range:
 	or	L1_CACHE_TAG_VALID,a1		# retain valid entries in the
 						# cache
 
-mn10300_dcache_flush_range_loop:
+mn10300_local_dcache_flush_range_loop:
 	mov	a1,(L1_CACHE_WAYDISP*0,a0)	# conditionally purge this line
 						# all ways
 
@@ -109,55 +140,80 @@ mn10300_dcache_flush_range_loop:
 	add	L1_CACHE_BYTES,a1
 	and	~L1_CACHE_WAYDISP,a0		# make sure way stay on way 0
 	add	-1,d1
-	bne	mn10300_dcache_flush_range_loop
+	bne	mn10300_local_dcache_flush_range_loop
+
+mn10300_local_dcache_flush_range_end:
+	ret	[d2],4
 
-mn10300_dcache_flush_range_end:
-	ret	[d2,d3],8
+	.size	mn10300_local_dcache_flush_page,.-mn10300_local_dcache_flush_page
+	.size	mn10300_local_dcache_flush_range,.-mn10300_local_dcache_flush_range
+	.size	mn10300_local_dcache_flush_range2,.-mn10300_local_dcache_flush_range2
 
 ###############################################################################
 #
-# void mn10300_dcache_flush_inv(void)
+# void mn10300_local_dcache_flush_inv(void)
 # Flush the entire data cache and invalidate all entries
 #
 ###############################################################################
 	ALIGN
-mn10300_dcache_flush_inv:
+	.globl	mn10300_local_dcache_flush_inv
+	.type	mn10300_local_dcache_flush_inv,@function
+mn10300_local_dcache_flush_inv:
 	movhu	(CHCTR),d0
 	btst	CHCTR_DCEN,d0
-	beq	mn10300_dcache_flush_inv_end
+	beq	mn10300_local_dcache_flush_inv_end
 
-	# hit each line in the dcache with an unconditional purge
-	mov	DCACHE_PURGE(0,0),a1		# dcache purge request address
-	mov	L1_CACHE_NWAYS*L1_CACHE_NENTRIES,d1  # total number of entries
+	mov	L1_CACHE_NENTRIES,d1
+	clr	a1
 
-mn10300_dcache_flush_inv_loop:
-	mov	(a1),d0				# unconditional purge
+mn10300_local_dcache_flush_inv_loop:
+	mov	(DCACHE_PURGE_WAY0(0),a1),d0	# unconditional purge
+	mov	(DCACHE_PURGE_WAY1(0),a1),d0	# unconditional purge
+	mov	(DCACHE_PURGE_WAY2(0),a1),d0	# unconditional purge
+	mov	(DCACHE_PURGE_WAY3(0),a1),d0	# unconditional purge
 
 	add	L1_CACHE_BYTES,a1
 	add	-1,d1
-	bne	mn10300_dcache_flush_inv_loop
+	bne	mn10300_local_dcache_flush_inv_loop
 
-mn10300_dcache_flush_inv_end:
+mn10300_local_dcache_flush_inv_end:
 	ret	[],0
+	.size	mn10300_local_dcache_flush_inv,.-mn10300_local_dcache_flush_inv
 
 ###############################################################################
 #
 # void mn10300_dcache_flush_inv_page(unsigned long start)
 # void mn10300_dcache_flush_inv_range(unsigned long start, unsigned long end)
 # void mn10300_dcache_flush_inv_range2(unsigned long start, unsigned long size)
 # Flush and invalidate a range of addresses on a page in the dcache
 #
 ###############################################################################
 	ALIGN
-mn10300_dcache_flush_inv_page:
+	.globl	mn10300_local_dcache_flush_inv_page
+	.globl	mn10300_local_dcache_flush_inv_range
+	.globl	mn10300_local_dcache_flush_inv_range2
+	.type	mn10300_local_dcache_flush_inv_page,@function
+	.type	mn10300_local_dcache_flush_inv_range,@function
+	.type	mn10300_local_dcache_flush_inv_range2,@function
+mn10300_local_dcache_flush_inv_page:
+	and	~(PAGE_SIZE-1),d0
 	mov	PAGE_SIZE,d1
-mn10300_dcache_flush_inv_range2:
+mn10300_local_dcache_flush_inv_range2:
 	add	d0,d1
-mn10300_dcache_flush_inv_range:
-	movm	[d2,d3],(sp)
+mn10300_local_dcache_flush_inv_range:
+	movm	[d2],(sp)
+
 	movhu	(CHCTR),d2
 	btst	CHCTR_DCEN,d2
-	beq	mn10300_dcache_flush_inv_range_end
+	beq	mn10300_local_dcache_flush_inv_range_end
+
+	sub	d0,d1,a0
+	cmp	MN10300_DCACHE_FLUSH_INV_BORDER,a0
+	ble	1f
+
+	movm	(sp),[d2]
+	bra	mn10300_local_dcache_flush_inv
+1:
 
 	and	L1_CACHE_TAG_ADDRESS|L1_CACHE_TAG_ENTRY,d0	# round start
 								# addr down
@@ -178,7 +234,7 @@ mn10300_dcache_flush_inv_range:
 	lsr	L1_CACHE_SHIFT,d1		# total number of entries to
 						# examine
 
-mn10300_dcache_flush_inv_range_loop:
+mn10300_local_dcache_flush_inv_range_loop:
 	mov	a1,(L1_CACHE_WAYDISP*0,a0)	# conditionally purge this line
 						# in all ways
 
@@ -186,7 +242,10 @@ mn10300_dcache_flush_inv_range_loop:
 	add	L1_CACHE_BYTES,a1
 	and	~L1_CACHE_WAYDISP,a0		# make sure way stay on way 0
 	add	-1,d1
-	bne	mn10300_dcache_flush_inv_range_loop
+	bne	mn10300_local_dcache_flush_inv_range_loop
 
-mn10300_dcache_flush_inv_range_end:
-	ret	[d2,d3],8
+mn10300_local_dcache_flush_inv_range_end:
+	ret	[d2],4
+	.size	mn10300_local_dcache_flush_inv_page,.-mn10300_local_dcache_flush_inv_page
+	.size	mn10300_local_dcache_flush_inv_range,.-mn10300_local_dcache_flush_inv_range
+	.size	mn10300_local_dcache_flush_inv_range2,.-mn10300_local_dcache_flush_inv_range2
@@ -44,41 +60,56 @@ mn10300_dcache_flush:
 ###############################################################################
 #
-# void mn10300_dcache_flush_page(unsigned long start)
-# void mn10300_dcache_flush_range(unsigned long start, unsigned long end)
-# void mn10300_dcache_flush_range2(unsigned long start, unsigned long size)
+# void mn10300_local_dcache_flush_page(unsigned long start)
+# void mn10300_local_dcache_flush_range(unsigned long start, unsigned long end)
+# void mn10300_local_dcache_flush_range2(unsigned long start, unsigned long size)
 # Flush a range of addresses on a page in the dcache
 #
@@ -109,55 +140,80 @@ mn10300_dcache_flush_range_loop:
 ###############################################################################
 #
-# void mn10300_dcache_flush_inv_page(unsigned long start)
-# void mn10300_dcache_flush_inv_range(unsigned long start, unsigned long end)
-# void mn10300_dcache_flush_inv_range2(unsigned long start, unsigned long size)
+# void mn10300_local_dcache_flush_inv_page(unsigned long start)
+# void mn10300_local_dcache_flush_inv_range(unsigned long start, unsigned long end)
+# void mn10300_local_dcache_flush_inv_range2(unsigned long start, unsigned long size)
 # Flush and invalidate a range of addresses on a page in the dcache
 #